/*
 * Dan Aloni <da-x@colinux.org>, 2005 (c)
 *
 * The code is licensed under the GPL. See the COPYING file at
 * the root directory.
 */

#ifndef __UPM_H__
#define __UPM_H__

#include <sys/mman.h>

#ifdef __cplusplus
extern "C" {
#endif

/*
 * mmap_offset() - creates a file descriptor that provides a read-only or COW mmap() 
 * capability for the passed file descriptor, but with an applied byte offset which 
 * can be page unaligned.
 *
 * The creation and of the new fd with regard to the old fd has the same semantics as 
 * dup() has.
 *
 * -1 is returned on errors, and errno is set appropriately.
 */

int mmap_offset(int fd, unsigned long offset);

/*
 * mmap() wrapper that supports unaligned offsets. See mmap().
 */
void *mmap_unaligned(void *start, size_t length, int prot, int flags, int fd, off_t offset);

#ifdef __cplusplus
};
#endif

#endif
