/*
 * Dan Aloni <da-x@colinux.org>, 2005 (c)
 *
 * The code is licensed under the GPL. See the COPYING file at
 * the root directory.
 */

/*
 * upmfs library functions.
 */

#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <asm/errno.h>
#include <asm/page.h>

#include <errno.h>
#include <linux/upmfs.h>
#include <upm.h>

int mmap_offset(int fd, unsigned long offset) 
{
	int upmfs_fd, ret;
	struct upm_io_unaligned_map io;	
	
	upmfs_fd = open("/proc/upmfs/ioctl", O_RDONLY);
	if (upmfs_fd == -1) {
		printf("upmfs not loaded\n");
		errno = ENODEV;
		return -1;
	}

	io.old_fd = fd;
	io.new_fd = -1;
	io.offset = offset;

	ret = ioctl(upmfs_fd, UPM_IO_UNALIGNED_MAP, &io);
	close(upmfs_fd);

	if (ret == -1) {
		printf("map_offset failed\n");
		goto exit;
	}

	ret = io.new_fd;

exit:
	return ret;
}

void *mmap_unaligned(void *start, size_t length, int prot, int flags, int fd, off_t offset)
{
	int close_fd = 0;
	void *ret = NULL;

	/* check if the offset is not page aligned */
	if (offset & ~PAGE_MASK) {
		/* read-only or COW - not supporting storage back to the file */
		if (flags & MAP_SHARED) {
			errno = EINVAL;
			return MAP_FAILED;
		}

		/* use upm and compensate for offset */
		fd = mmap_offset(fd, offset & ~PAGE_MASK);
		if (fd == -1)
			return MAP_FAILED;

		offset &= PAGE_MASK;
		close_fd = 1;
	}

	ret = mmap(start, length, prot, flags, fd, offset);

	if (close_fd)
		close(fd);

	return ret;
}
