/*
 * Dan Aloni <da-x@colinux.org>, 2005 (c)
 *
 * The code is licensed under the GPL. See the COPYING file at
 * the root directory.
 */

#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <stdlib.h>

#include <upm.h>

unsigned long mapping_size = 0x4000;
unsigned long mapping_offset = 0x4000;
const char *filename = "testfile";

int create_test_file(void)
{
	int fd, i, j;
	int num_blocks = 0x100;
	char block[0x1000];

	printf("generating test file\n");

	fd = open(filename, O_RDWR | O_CREAT, 0666);
	if (!fd) 
		return -1;

	for (j=0; j < num_blocks; j++) {
		for (i=0; i < sizeof(block); i++)
			block[i] = rand();
		write(fd, block, sizeof(block));
	}

	close(fd);
	return 0;	
}

int main(int argc, char **argv)
{
	int fd, new_fd;
	char *addr, *addr_unaligned;
	int i, j, result;

	result = create_test_file();
	if (result) {
		printf("error creating test file: %d\n", result);
		return -1;
	}

	fd = open(filename, O_RDONLY);
	if (fd == -1) {
		perror("open");
		return -1;
	}

	addr = mmap(NULL, mapping_size, PROT_READ | PROT_EXEC, MAP_PRIVATE, fd, mapping_offset);
	if (addr == MAP_FAILED) {
		perror("mmap");
		printf("unable to map %s\n", filename);
		return -1;
	}

	printf("normal mapping at %x\n", addr);

	for (i=0; i < 0x2000; i++) {
		new_fd = mmap_offset(fd, i); 
		if (new_fd == -1) {
			printf("aborting\n");
			return -1;
		}
		
		addr_unaligned = mmap(NULL, mapping_size, PROT_READ | PROT_EXEC, MAP_SHARED, new_fd, mapping_offset);
		if (addr_unaligned == MAP_FAILED) {
			printf("failure mapping at alignment %d\n", i);
			return -1;
		}

		for (j=0; j < mapping_size-i; j++) {
			if (addr_unaligned[j] != addr[i+j]) {
				printf("error at %d, %d\n", i, j);
				return -1;
			}
		}
		
		munmap(addr_unaligned, mapping_size);
		close(new_fd);
	}

	printf("all okay\n");

	return 0;
}
